/*
 *  polyConv.cpp
 *  fp
 *
 *  Created by Robert Delaney on 4/27/13.
 *  Copyright 2013 Bob Delaney's Science Software. All rights reserved.
 *
 */

#include "polyConv.h"
#include "stringUtils.h"
#include "mb2Str.h"
#include "polyMath.h"

static bf getTheFraction(const char *theField)
{
	bf		z;
	char	*fracStr, ch;
	long	i, length, index;
	
	length = strlen(theField);
	if(!length)
		return z;
	
	fracStr = (char*)malloc((length+1)*sizeof(char));
	
	for(i=0;i<length;i++)
	{
		index = i;
		ch = theField[i];
		if(!(ch=='/'))
			fracStr[i]= ch;
		else {
			break;
		}
	}
	if(ch=='/')
		fracStr[index] = 0;
	else {
		fracStr[index+1] = 0;
	}

	mbConvFromStr(z.num, fracStr);
	if(index==(length-1))
	{
		z.den = 1;
		free(fracStr);
		return z;
	}
	
	for(i=index+1;i<length;i++)
	{
		fracStr[i-index-1] = theField[i];
	}
	fracStr[length-index-1] = 0;
	
	mbConvFromStr(z.den, fracStr);
	
	free(fracStr);
	
	return z;
	
}/* getTheFraction */

// polyStr of form "coeff 0, coeff 1, ... , coeff deg"
bool polyConvFromString(poly& z, const char *polyStr)
{
	char	*theField;
	long	i;
	
	if(!polyStr || !strlen(polyStr))
		return false;
	
	z.deg = CountFields(polyStr, ",") - 1;
	z.array = (bf*)malloc((z.deg+1)*sizeof(bf));
	
	for(i=0;i<=z.deg;i++)
	{
		theField = NthField(polyStr, ",", i+1);
		z.array[i] = getTheFraction(theField);
		free(theField);
		if(!z.array[i].den)
			return false;
	}
	
	z = polyNormalize(z); // if user puts in high order zeroes!

	return true;
	
}/* polyConvFromString */


void polyConvToString(char*& xString, const poly& x)
{
	char	*numString, *denString;
	long	i, numLength, denLength, currLength;
	
	if(x.deg<0)
	{
		xString = NULL;
		return;
	}
	
	currLength = 1;
	xString = (char*)malloc(currLength*sizeof(char));
	xString[0] = 0;
	
	i = 0;
	numString = mb2Str(x.array[i].num);
	numLength = strlen(numString);
	if(x.array[i].den!=1)
	{
		denString = mb2Str(x.array[i].den);
		denLength = strlen(denString);
		currLength = currLength + numLength + denLength + 3; // 3 is for ", " and "/"
		xString = (char*)realloc(xString, currLength);
		if(xString==NULL)
			return;
		strcpy(xString, numString);
		free(numString);
		strcat(xString, "/");
		strcat(xString, denString);
		free(denString);
	}
	else
	{
		currLength = currLength + numLength + 2; // 2 is for ", "
		xString = (char*)realloc(xString, currLength);
		if(xString==NULL)
			return;
		strcpy(xString, numString);
		free(numString);
	}
	
	for(i=1;i<=x.deg;i++)
	{
		numString = mb2Str(x.array[i].num);
		numLength = strlen(numString);
		if(x.array[i].den!=1)
		{
			denString = mb2Str(x.array[i].den);
			denLength = strlen(denString);
			currLength = currLength + numLength + denLength + 3; // 3 is for ", " and "/"
			xString = (char*)realloc(xString, currLength);
			if(xString==NULL)
				return;
			strcat(xString, ", ");
			strcat(xString, numString);
			free(numString);
			strcat(xString, "/");
			strcat(xString, denString);
			free(denString);
		}
		else
		{
			currLength = currLength + numLength + 2; // 2 is for ", "
			xString = (char*)realloc(xString, currLength);
			if(xString==NULL)
				return;
			strcat(xString, ", ");
			strcat(xString, numString);
			free(numString);
		}

	}
	
}/* polyConvToString */

